#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on Mar 6, 2012

@author: Greivin Lopez
'''

# Copyright (c) 2012 The Skuë Project
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

# ***** Python built-in modules *****
import json

__author__ = "Greivin Lopez"
__copyright__ = u"Copyright 2012, The Skuë Project"
__credits__ = ["Greivin Lopez"]
__license__ = "MIT"
__version__ = "1"
__maintainer__ = "Greivin Lopez"
__email__ = "greivin.lopez@gmail.com"
__status__ = "Development"


#===============================================================================
# ResourceJSONEncoder
#===============================================================================
class ResourceJSONEncoder(json.JSONEncoder):
    '''Custom JSON Encoder to serialize complex objects'''
    def default(self, obj):
        if isinstance(obj, ResourceJSONRepresentation):
            response = {}
            for field in dir(obj):
                if not field.startswith('_') and not field in obj.exclude:
                    response[field] = getattr(obj, field)
            return response
        return json.JSONEncoder.default(self, obj)

#===============================================================================
# ResourceJSONRepresentation
#===============================================================================
class ResourceJSONRepresentation(object):
    '''Base class to extend when creating objects to be serialized as JSON
    for HTTP response bodies'''
    def __init__(self, object_type):
        self._object_type = object_type
        # Members to exclude when encoding
        self.exclude = ['exclude', 'as_json', 'is_http_error', 'get_localized']

    def as_json(self):
        return json.dumps(self, cls=ResourceJSONEncoder)

    @property
    def is_http_error(self):
        return False

    def get_localized(self, obj, field_name, language):
        if language != 'en':
            localized_field = ''.join([field_name, '_', language])
            if hasattr(obj, localized_field):
                return getattr(obj, localized_field)
        return getattr(obj, field_name)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on Apr 3, 2012

@author: Greivin Lopez
'''

# Copyright (c) 2012 The Skuë Project
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

# ***** Application modules *****
from .json.utils import ResourceJSONRepresentation
from .rest.api import RepresentationType as ContentType

__author__ = "Greivin Lopez"
__copyright__ = u"Copyright 2012, The Skuë Project"
__credits__ = ["Greivin Lopez"]
__license__ = "MIT"
__version__ = "1"
__maintainer__ = "Greivin Lopez"
__email__ = "greivin.lopez@gmail.com"
__status__ = "Development"


#===============================================================================
# HandlerHttpResponse
#===============================================================================
class HandlerHttpResponse(object):
    """Represents an HTTP response wrapper for the API Server"""

    # A dictionary with the headers to include in the response
    headers = {}
    # The HTTP status code to return
    status_code = None
    # The body for the response: Assume to be a ResponseObject
    body = None
    # The intended "Content-Type" of the response
    _content_type = None

    @property
    def content_type(self):
        """The intended 'Content-Type' of the response"""
        return self._content_type

    @content_type.setter
    def content_type(self, value):
        self._content_type = value
        self.headers['Content-Type'] = value

    def __init__(self, status_code, content_type, body="", headers={}):
        """Creates a new HandlerHttpResponse with the given arguments

        Args:
          status_code: The HTTP status code for the response
          content_type: The intended "Content-Type" of the response
          body: A ResourceResponse to use as the body for the response
          headers: A dictionary with the HTTP headers of the response
        """
        self.status_code = status_code
        self.headers = headers
        self.content_type = content_type
        self.body = body

    def write_body(self):
        """Writes out a representation of the body of this HTTP response
        according to the response's Content-Type
        """
        if self.content_type == ContentType.JSON:
            return self.body.as_json()
        else:
            return self.body


#===============================================================================
# CommonResponse
#===============================================================================
class CommonResponse(object):

    @classmethod
    def success(cls, body, content_type=ContentType.JSON):
        http_response = HandlerHttpResponse(status_code=200,
                                            content_type=content_type,
                                            body=body)
        return http_response

    @classmethod
    def simple_success(cls, message, content_type=ContentType.JSON):
        body = ResourceJSONRepresentation('Success')
        body.status = "OK"
        body.message = message
        http_response = HandlerHttpResponse(status_code=200,
                                            content_type=content_type,
                                            body=body)
        return http_response

    @classmethod
    def options(cls, allowed_methods, body, content_type=ContentType.JSON):
        http_response = HandlerHttpResponse(status_code=200,
                                            content_type=content_type,
                                            body=body,
                                            headers={"Allow": allowed_methods})
        return http_response

    @classmethod
    def method_not_allowed(cls, allowed_methods,
                           content_type=ContentType.JSON):
        body = ResourceJSONRepresentation('MethodNotAllowed')
        body.status = "Error"
        body.message = "Method Not Allowed"
        http_response = HandlerHttpResponse(status_code=405,
                                            content_type=content_type,
                                            body=body,
                                            headers={"Allow": allowed_methods})
        return http_response

    @classmethod
    def resource_not_found(cls, content_type=ContentType.JSON):
        body = ResourceJSONRepresentation('ResourceNotFound')
        body.status = "Error"
        body.message = "The resource could not be found"
        http_response = HandlerHttpResponse(status_code=404,
                                            content_type=content_type,
                                            body=body)
        return http_response

    @classmethod
    def resource_created(cls, resource_uri, content_type=ContentType.JSON):
        body = ResourceJSONRepresentation('Success')
        body.status = "OK"
        body.message = "Successfully created"
        body.uri = resource_uri
        http_response = HandlerHttpResponse(
            status_code=201,
            content_type=content_type,
            body=body,
            headers={"Location": resource_uri.encode('ascii', 'ignore')})
        return http_response

    @classmethod
    def resource_creation_queued(cls, content_type=ContentType.JSON):
        body = ResourceJSONRepresentation('Success')
        body.status = "OK"
        body.message = "Successfully queued for creation"
        http_response = HandlerHttpResponse(status_code=202,
                                            content_type=content_type,
                                            body=body)
        return http_response

    @classmethod
    def not_acceptable(cls, content_type=ContentType.JSON):
        body = ResourceJSONRepresentation('NotAcceptable')
        body.status = "Error"
        body.message = "Not Acceptable"
        http_response = HandlerHttpResponse(status_code=406,
                                            content_type=content_type,
                                            body=body)
        return http_response

    @classmethod
    def custom_validation_error(cls, message, content_type=ContentType.JSON):
        body = ResourceJSONRepresentation('CustomError')
        body.status = "Error"
        body.message = message
        http_response = HandlerHttpResponse(status_code=400,
                                            content_type=content_type,
                                            body=body)
        return http_response

    @classmethod
    def custom_server_error(cls, message, content_type=ContentType.JSON):
        body = ResourceJSONRepresentation('CustomError')
        body.status = "Error"
        body.message = message
        http_response = HandlerHttpResponse(status_code=500,
                                            content_type=content_type,
                                            body=body)
        return http_response



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on Apr 6, 2012

@author: Greivin Lopez
'''

# Copyright (c) 2012 The Skuë Project
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

# ***** Python built-in modules *****
import traceback
import importlib
from types import StringTypes

# ***** Pyramid modules *****
from pyramid.response import Response
from pyramid.httpexceptions import HTTPMethodNotAllowed

# ***** Application modules *****
from .api import ApiDescription
from .api import RepresentationType as ContentType
from .api import ResourceOptionsJSONRepresentation
from .api import RestApiDocJSONRepresentation
from ..errors import ResponseError, ParameterMissedError, NotAcceptableError
from ..http import CommonResponse
from ..utils import parse_body

__author__ = "Greivin Lopez"
__copyright__ = u"Copyright 2012, The Skuë Project"
__credits__ = ["Greivin Lopez"]
__license__ = "MIT"
__version__ = "1"
__maintainer__ = "Greivin Lopez"
__email__ = "greivin.lopez@gmail.com"
__status__ = "Development"


SKUE_CACHE = {}


class BaseHandler(object):
    """Base handler with ACL management.

    Inherit and define the desired methods,
    then register views and set permissions.

    """

    def __init__(self, request):
        self.request = request
        self.response = Response()

    def __call__(self):
        """Dispatch the request."""
        method = self.request.method
        try:
            return getattr(self, method.lower())()
        except (NotImplementedError):
            raise HTTPMethodNotAllowed

    def get(self):
        raise NotImplementedError

    def options(self):
        raise NotImplementedError

    def post(self):
        raise NotImplementedError

    def put(self):
        raise NotImplementedError

    def delete(self):
        raise NotImplementedError


#===============================================================================
# RestResource
#===============================================================================
class RestResource(BaseHandler):
    """The request handler for RESTful controllers."""

    # A dictionary with the parameters of the request regardless of method
    payload = None
    # The list of the required parameters of the request
    required = []
    # The list of the optional parameters of the request
    optional = []
    # String representation of the HTTP method of the current request
    http_method = ''
    # The value of the 'HTTP_USER_AGENT' header
    http_user_agent = ''
    # The RestResourceDescription object that describes this controller options
    resource_description = None
    # The host name of the server
    host_name = None
    # The language the client prefer for the response
    language = 'en'

    def handle_request(self, method, *args, **kwargs):
        """Method that calls the send_response method with the response of
        the resource specific call.

        This function is called with the specific function for the REST
        operation.

        Args:
          method: the method that executes the REST operation
        """
        try:
            self.__entrance()
            return self.send_response(method(*args, **kwargs))
        except ResponseError, error:
            return self.send_response(error.get_http_response())
        except Exception as error:
            # self.logger.exception('An unexpected error has occur')
            return self.handle_unexpected_error(error)

    def __entrance(self):
        """First method executed by every API request

        Will perform common requests stuff such as populating
        internal variables describing the request and loading
        the input parameters and doing automatic validation.
        """
        try:
            # Set the HTTP Method
            self.http_method = self.request.method
            # Set the host name
            self.host_name = self.request.host_url
            # Set the user agent
            user_agent = self.request.user_agent

            if user_agent is not None and isinstance(user_agent, StringTypes):
                self.http_user_agent = user_agent

            # Set the intended response representation
            accepted_format = self.request.headers.get('Accept');
            if accepted_format is not None and not '*/*' in accepted_format:
                self.content_type = accepted_format
            else:
                self.content_type = ContentType.JSON

            # Get the self description of the Resource
            self.resource_description = self.__get_self_description()
            # Load the parameters sent in the HTTP request
            self.__load_parameters()
            # Deduce the expected parameters from description
            self.__deduce_expected_parameters()
            # Validate the request
            self.__validate_request()
            # Validate the parameters
            self.__validate_parameters()
        except Exception as error:
            #self.logger.exception('An unexpected error has occur')
            return self.handle_unexpected_error(error)

    def __get_self_description(self):
        """Gets the description of the resource provided by inheritors.
        It not always create the result from scratch it will first check in
        memory cache object.

        """
        if self.resource_description is not None:
            return self.resource_description
        else:
            method_key = ''.join([self.request.path, '-', self.http_method])
            resource_description = SKUE_CACHE.get(method_key)
            if resource_description is not None:
                self.resource_description = resource_description
                return self.resource_description
            else:
                self.resource_description = self.describe_resource()
                SKUE_CACHE[method_key] = self.resource_description
                return self.resource_description

    def __get_payload(self):
        """Populates payload dictionary from method arguments.

        Retrieve the arguments from the HTTP method and populates
        the payload dictionary of this instance.

        Returns:
          A dict structure with the HTTP request arguments and its values
        """
        payload = {}
        if self.http_method in ('POST', 'PUT'):
            payload = parse_body(self.request.body)
        else:
            for key, value in self.request.params.items():
                payload[key] = value
        return payload

    def __load_parameters(self):
        """Load the parameters of the request.

        Validates the parameters to find missed required parameters or
        unexpected parameters received.

        Raises:
          ParameterMissedError: One of the required parameters is not present
          NotExpectedParameterError: One of the received parameters was not registered
        """
        # Clean the parameter sets.. (unregister previous parameters)
        self.required = []
        self.optional = []

        # Get payload function populates the payload dictionary
        self.payload = self.__get_payload()

    def __deduce_expected_parameters(self):
        """This method it's called every time a request is made.

        It will deduce the different allowed parameters for the current
        request method and which parameters are expected as required
        for the controller to handle validation automatically.
        """
        self_description = self.resource_description
        if self_description is not None:
            methods = self_description.methods
            if methods is not None:
                for method in methods:
                    if method.method == self.http_method:
                        for parameter in method.parameters:
                            if parameter.is_required:
                                self.required.append(parameter.name)
                            else:
                                self.optional.append(parameter.name)

    def __validate_parameters(self):
        """Validate web request parameters.

        Validates the registered parameters against the received parameters
        to determine which required parameters are missed and if there is
        not expected parameters it will remove them from payload.

        Raises:
          ParameterMissedError: One of the required parameters is not present
        """
        # Sets the language of the request if provided
        if 'lang' in self.payload:
            self.language = self.payload['lang']
            del self.payload['lang']

        # Create the needed sets to work with
        required_set = set(self.required)
        optional_set = set(self.optional)
        received_set = set(self.payload.keys())

        # Validate required parameters
        fail_required = required_set - received_set
        if fail_required:
            for expected_parameter in fail_required:
                raise ParameterMissedError(parameter=expected_parameter)

        # Find any invalid params and remove them from self.payload
        valid_set = required_set.union(optional_set)
        invalid_params = received_set - valid_set

        for param in invalid_params:
            del self.payload[param]


    def __validate_request(self):
        """Validates the HTTP request to ensure the ability of
        of this web handler to fulfill the expectations of the
        client with it's response.
        """
        self_description = self.resource_description
        if self_description is not None:
            methods = self_description.methods
            if methods is not None:
                for method in methods:
                    if method.method == self.http_method:
                        if not self.content_type in method.representations:
                            raise NotAcceptableError(method.representations)

    def options_for_resource(self, *args, **kwargs):
        """Retrieves the information related the communication options
        associated to a particular resource

        Returns:
          A self description of the resource in JSON representation
        """
        response_body = ResourceOptionsJSONRepresentation(self.resource_description)
        return CommonResponse.options(self.get_allowed_methods(), response_body)

    def get_allowed_methods(self):
        """Returns a string with the list of HTTP allowed methods"""
        allowed_methods = ""
        if self.resource_description.methods is not None:
            methods_list = [method.method for method in self.resource_description.methods]
            allowed_methods = ', '.join(methods_list)
        return allowed_methods

    def send_response(self, handler_response):
        """Writes an output to the API client with the given response

        Args:
          handler_response: An instance of HandlerHttpResponse that contains
          the data for the response.
        """
        self.response.status_int = handler_response.status_code
        self.response.headerlist = handler_response.headers.iteritems()
        self.response.body = handler_response.write_body()
        return self.response

    def handle_unexpected_error(self, error):
        """Handles the given unexpected error.

        Unexpected error is something that should return a 500 status code
        and also write to log the context and error details.

        Args:
          error: The unexpected error to handle.

        Returns:
          An HTTP 500 error response if not in development environment.
        """
        # return self.error(500)
        raise error.__class__(traceback.format_exc(error))

    #===========================================================================
    # The HTTP Method Handlers
    #===========================================================================
    def get(self, *args, **kwargs):
        """Handler implementation of an HTTP GET method."""
        return self.handle_request(self.read_resource, *args, **kwargs)

    def options(self, *args, **kwargs):
        """Handler implementation of an HTTP OPTIONS method.
        """
        return self.handle_request(self.options_for_resource, *args, **kwargs)

    def post(self, *args, **kwargs):
        """Handler implementation of an HTTP POST method."""
        return self.handle_request(self.create_resource, *args, **kwargs)

    def put(self, *args, **kwargs):
        """Handler implementation of an HTTP PUT method."""
        return self.handle_request(self.update_resource, *args, **kwargs)

    def delete(self, *args, **kwargs):
        """Handler implementation of an HTTP DELETE method."""
        return self.handle_request(self.delete_resource, *args, **kwargs)

    #===========================================================================
    # Methods for inheritors
    #===========================================================================
    def create_resource(self, *args, **kwargs):
        """This method should allow users to create new resources.

        Should be implemented by inheritors of this class to create
        new instances of the resource.
        """
        return CommonResponse.method_not_allowed(self.get_allowed_methods())

    def read_resource(self, *args, **kwargs):
        """Retrieve the information of the current resource.
        """
        return CommonResponse.method_not_allowed(self.get_allowed_methods())

    def update_resource(self, *args, **kwargs):
        """Updates the current resource instance.
        """
        return CommonResponse.method_not_allowed(self.get_allowed_methods())

    def delete_resource(self, *args, **kwargs):
        """Removes the resource associated with the given value.
        """
        return CommonResponse.method_not_allowed(self.get_allowed_methods())


#===============================================================================
# DocumentResource
#===============================================================================
class DocumentResource(RestResource):
    """Represents a Document resource. A Document resource is an archetype
    to represent a singular concept.
    Individual items that could be part of a Collection resource.
    """
    pass

#===============================================================================
# StoreDocumentResource
#===============================================================================
class StoreDocumentResource(RestResource):
    """Represents a Document resource. A Document resource is an archetype
    to represent a singular concept.

    The store document differs from the regular document resource because
    it represents individual elements that are part of a Store resource
    so there are some differences in the way this kind of resource should
    handle the HTTP methods.

    Inheritors MUST handle the exists method to indicate if there is
    an existing resource with a provided identification value in order
    to the object to create itself or return it's representation.
    """
    # Will store the URI for a newly created item
    _new_resource_uri = ""

    @property
    def new_resource_uri(self):
        """Return the newly created resource URI"""
        return self._new_resource_uri

    def put(self, *args, **kwargs):
        """Handler implementation of an HTTP PUT method."""
        if len(args) > 0 or len(kwargs) > 0:
            identifier = args[0] if len(args) else kwargs.values()[0]
            if self.exists(identifier):
                # update an existing resource
                super(StoreDocumentResource, self).handle_request(self.update_resource, *args, **kwargs)
            else:
                # create a new resource
                self._new_resource_uri = self.request.path
                super(StoreDocumentResource, self).handle_request(self.create_resource, *args, **kwargs)


    def exists(self, identifier):
        """Gets a value to indicate if the given identifier it's already
        associated to an existing resource.
        This method MUST be implemented for inheritors of this class.

        Args:
          identifier: A value that uniquely identify a resource

        Returns:
          True if the identifier is associated to an existing resource or
          False otherwise.
        """
        return False

#===============================================================================
# CollectionResource
#===============================================================================
class CollectionResource(RestResource):
    """Represents a Collection resource. A Collection resource is a list of
    items or other resources handled by the Server.
    """
    # Will store the offset and limit for pagination in GET requests
    _offset = 0
    _count = 100

    @property
    def offset(self):
        """Return the starting point when retrieving results in a GET"""
        return self._offset

    @property
    def count(self):
        """Return the maximum number of items to return results in a GET"""
        return self._count

    def get(self, *args, **kwargs):
        """Handler implementation of an HTTP GET method."""
        self._offset = int(self.request.params.get("offset", default_value="0"))
        self._count  = int(self.request.params.get("count", default_value="100"))
        super(CollectionResource, self).handle_request(self.read_resource, *args, **kwargs)


#===============================================================================
# StoreResource
#===============================================================================
class StoreResource(RestResource):
    """Represents an Store resource. An Store resource is a collection of
    resources handled by the client.
    """
    def post(self, *args, **kwargs):
        """Handler implementation of an HTTP POST method."""
        super(StoreResource, self).handle_request(self.post_not_allowed, *args, **kwargs)

    def post_not_allowed(self, *args, **kwargs):
        """Returns a response to indicate that the POST method should not
        be allowed on resources of type Store"""
        return CommonResponse.method_not_allowed(self.get_allowed_methods())


#===============================================================================
# ControllerResource
#===============================================================================
class ControllerResource(RestResource):
    """Represents a Controller resource. A Controller resource is a model
    to the concept of a procedure. Some action that is going to be executed
    over other resources but doesn't quite map to the CRUD model.
    """
    def post(self, *args, **kwargs):
        """Handler implementation of an HTTP POST method."""
        super(ControllerResource, self).handle_request(self.execute, *args, **kwargs)

    def execute(self, *args, **kwargs):
        """Performs the execution of the controller's action"""
        return CommonResponse.method_not_allowed(self.get_allowed_methods())


#===============================================================================
# ApiDocumentationResource
#===============================================================================
class ApiDocumentationResource(BaseHandler):
    """A new kind of resource that allows you to self-document your entire API"""

    #===========================================================================
    # The HTTP Method Handlers
    #===========================================================================
    def get(self, *args, **kwargs):
        """Handler implementation of an HTTP GET method."""
        return self.send_response(CommonResponse.method_not_allowed('OPTIONS'))

    def options(self, *args, **kwargs):
        """Handler implementation of an HTTP OPTIONS method.
        """
        self.create_api_documentation(*args, **kwargs)

    def post(self, *args, **kwargs):
        """Handler implementation of an HTTP POST method."""
        return self.send_response(CommonResponse.method_not_allowed('OPTIONS'))

    def put(self, *args, **kwargs):
        """Handler implementation of an HTTP PUT method."""
        return self.send_response(CommonResponse.method_not_allowed('OPTIONS'))

    def delete(self, *args, **kwargs):
        """Handler implementation of an HTTP DELETE method."""
        return self.send_response(CommonResponse.method_not_allowed('OPTIONS'))

    def send_response(self, handler_response):
        """Writes an output to the API client with the given response

        Args:
          handler_response: An instance of HandlerHttpResponse that contains
          the data for the response.
        """
        self.response.status_int = handler_response.status_code
        self.response.headerlist = handler_response.headers.iteritems()
        self.response.body = handler_response.write_body()
        return self.response

    def create_api_documentation(self, *args, **kwargs):
        """Creates the API documentation and return it to consumers"""
        import main
        resources = []
        for web_handler in main.API_HANDLERS:
            try:
                web_handler_instance = web_handler[1]()
            except TypeError:
                # ``webapp2.Route`` class is used for defining routing
                last_dot = web_handler.handler.rindex('.')
                handler_module = web_handler.handler[:last_dot]
                handler_class = web_handler.handler[last_dot + 1:]
                web_handler_instance = getattr(
                    importlib.import_module(handler_module), handler_class)()
            if isinstance(web_handler_instance, RestResource):
                resources.append(web_handler_instance.describe_resource())
            del web_handler_instance

        api_name = main.API_NAME if main.API_NAME else 'API Name Unknown'
        api_description = main.API_DESCRIPTION if main.API_DESCRIPTION else 'No description provided'

        api_documentation = ApiDescription(name=api_name,
                                           resources=resources,
                                           description=api_description)

        response_body = RestApiDocJSONRepresentation(api_documentation)
        return self.send_response(CommonResponse.options('OPTIONS', response_body))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on Apr 2, 2012

@author: Greivin Lopez
'''

# Copyright (c) 2012 The Skuë Project
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

# ***** Application modules *****
from .json.utils import ResourceJSONRepresentation
from .rest.api import RepresentationType as ContentType
from .http import HandlerHttpResponse

__author__ = "Greivin Lopez"
__copyright__ = u"Copyright 2012, The Skuë Project"
__credits__ = ["Greivin Lopez"]
__license__ = "MIT"
__version__ = "1"
__maintainer__ = "Greivin Lopez"
__email__ = "greivin.lopez@gmail.com"
__status__ = "Development"


#===============================================================================
# ResponseError
#===============================================================================
class ResponseError(Exception):
    """Represents a response error.

    This is the base class for all the response errors.
    Mainly errors we want to send to the user as HTTP error responses.
    All the server errors that should be presented to the requester client
    should inherit from this exception.
    """
    #@summary: The HTTP status code of the error
    code = None

    def __init__(self, code):
        """Creates a new instance of the ResponseError with the given
        arguments.

        Args:
          code: An alias to identify the error.
          source: The name of the class that generates the error.
        """
        self.code = code

    def __str__(self):
        """
        To provide a more custom display in our exceptions overriding this
        method allows us to return our dynamically constructed error message.
        """
        return self.message

    @property
    def message(self):
        """Text representation of this error.

        This represents the message to send to the client. It is a more
        detailed description of the error.
        It is intended to be returned as the description value of the
        error in the HTTP response.
        """
        return "An unknown error has occurred."

    def get_http_response(self, content_type=ContentType.JSON):
        """Creates an HandlerHttpResponse with JSON content to output
        this API handler error to a client.
        """
        body = ResourceJSONRepresentation('ApiHandledError')
        body.status = "Error"
        body.message = self.message
        http_response = HandlerHttpResponse(status_code=self.code,
                                           content_type=content_type,
                                           body=body)
        return http_response

#===============================================================================
# ParameterMissedError
#===============================================================================
class ParameterMissedError(ResponseError):
    """A required parameter is missed from an API call.
    """
    #@summary: The name of the parameter that is missed in the endpoint call.
    parameter = None

    @property
    def message(self):
        return "The %s parameter is missed." % self.parameter

    def __init__(self, parameter):
        ResponseError.__init__(self, code=400)
        self.parameter = parameter


#===============================================================================
# NotExpectedParameterError
#===============================================================================
class NotExpectedParameterError(ResponseError):
    """A given parameter is not registered as expected from an API call.
    """
    #@summary: The name of the parameter that was not expected but received.
    parameter = None

    @property
    def message(self):
        return "The %s parameter is not expected." % self.parameter

    def __init__(self, parameter):
        ResponseError.__init__(self, code=400)
        self.parameter = parameter

#===============================================================================
# InvalidParameterFormatError
#===============================================================================
class InvalidParameterFormatError(ResponseError):
    """An error to be raise when a given parameter does not conform with
    the expected formating of that parameter.
    """
    #@summary: The name of the parameter received with an invalid format.
    parameter = None

    @property
    def message(self):
        return "Invalid format for parameter %s." % self.parameter

    def __init__(self, parameter):
        ResponseError.__init__(self, code=400)
        self.parameter = parameter

#===============================================================================
# UnknownReferenceError
#===============================================================================
class UnknownReferenceError(ResponseError):
    """Response error when a given reference could not be found in the storage.

    Error response when a given parameter value representing a reference could
    not be found in the storage.
    """
    #@summary: The name of the parameter associated to this error
    name = None
    #@summary: The value of the parameter that could not be found in the storage
    value = None

    @property
    def message(self):
        return 'The %s=%s cannot be found into the storage.' % (self.name, self.value)

    def __init__(self, name, value):
        """Creates a new UnknownReferenceError."""
        ResponseError.__init__(self, code=404)
        self.name = name
        self.value = value

#===============================================================================
# NotAcceptableError
#===============================================================================
class NotAcceptableError(ResponseError):
    """Response error when detected a request that cannot be fullfil by the
    request handler.
    """
    #@summary: The supported representation for a resource
    supported_representations = ''

    @property
    def message(self):
        return 'The resource only supports the following representations: %s' % (self.supported_representations)

    def __init__(self, supported_representations):
        """Creates a new UnknownReferenceError."""
        ResponseError.__init__(self, code=406)
        self.supported_representations = supported_representations



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on Apr 3, 2012

@author: Greivin Lopez
'''

# Copyright (c) 2012 The Skuë Project
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

# ***** Python built-in modules *****
import urllib

__author__ = "Greivin Lopez"
__copyright__ = u"Copyright 2012, The Skuë Project"
__credits__ = ["Greivin Lopez"]
__license__ = "MIT"
__version__ = "1"
__maintainer__ = "Greivin Lopez"
__email__ = "greivin.lopez@gmail.com"
__status__ = "Development"


#===============================================================================
# parse_body
#===============================================================================
def parse_body(body):
    """Parses the given body from an HTTP request to retrieve arguments

    Args
      body: The body of the request. E.g. 'name=value&name2=value2'

    Returns:
      A dictionary with all the parameters extracted from the provided
      value. E.g. { 'name':'value', 'name2':'value2' }

    Note: All argument values will be strings.
    """
    output = {}
    if len(body) > 0:
        parameters = body.split('&')
        for parameter in parameters:
            parsed = parameter.split('=')
            if len(parsed) > 1:
                output[parsed[0]] = urllib.unquote_plus(parsed[1])
    return output

#===============================================================================
# restify
#===============================================================================
def restify(text):
    """Transform the given text to something that could be use as a section
    of a REST URI.

    Replaces white spaces with hyphens.

    Args:
      text: The text to be converted.

    Returns:
      A string value equivalent to the given text but suited for a REST URI.
    """
    text = text.replace(' ', '-').lower()
    return ''.join(char for char in text if char.isalnum() or char == '-')



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
CHANGES = open(os.path.join(here, 'CHANGES.rst')).read()

requires = [
    'pyramid>=1.5a2',
]

setup(name='pyramid_skue',
      version='0.1.0',
      description='pyramid_skue',
      long_description=README + '\n\n' + CHANGES,
      classifiers=[
          "Programming Language :: Python",
          "Programming Language :: Python :: 2",
          "Framework :: Pyramid",
          "Topic :: Internet :: WWW/HTTP",
      ],
      author='Cyril Panshine',
      author_email='kipanshi@gmail.com',
      url='https://github.com/kipanshi/pyramid_skue',
      keywords='web pyramid pylons rest api',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=requires,
      tests_require=requires,
      test_suite="pyramid_skue",
      )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on Apr 5, 2012

@author: Greivin Lopez
'''

# Copyright (c) 2012 The Skuë Project
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

# ***** Application modules *****
from ..json.utils import ResourceJSONRepresentation

__author__ = "Greivin Lopez"
__copyright__ = u"Copyright 2012, The Skuë Project"
__credits__ = ["Greivin Lopez"]
__license__ = "MIT"
__version__ = "1"
__maintainer__ = "Greivin Lopez"
__email__ = "greivin.lopez@gmail.com"
__status__ = "Development"


#===============================================================================
# RepresentationType
#===============================================================================
class RepresentationType(object):
    """A collection of constants to describe the different values that could
    be use as the Internet media type in Content-Type HTTP headers.

    Use this values as:
    from skue.rest.api import RepresentationType as MediaType

    if self.representation == MediaType.JSON:
        return output.as_json()

    @see: http://en.wikipedia.org/wiki/Internet_media_type
    """
    # @ivar JSON: JavaScript Object Notation media type
    JSON = "application/json"

    # @ivar ATOM: Atom feeds media type
    ATOM = "application/atom+xml"

    # @ivar TEXT: Textual data media type. Defined in RFC 2046 and RFC 3676
    TEXT = "text/plain"

    # @ivar XML: Extensible Markup Language (XML) media type
    XML = "application/xml"

    # @ivar JPEG:  JPEG JFIF image media type. Defined in RFC 2045 and RFC 2046
    JPEG = "image/jpeg"

    # @ivar PNG: Portable Network Graphics media type. Defined in RFC 2083
    PNG =  "image/png"


#===============================================================================
# ApiDescription
#===============================================================================
class ApiDescription(object):
    """Represents a self descriptive object to document a REST API"""
    _name = ''
    _resources = []
    _description = ''

    @property
    def name(self):
        """The name of the API"""
        return self._name

    @property
    def resources(self):
        """The list of the resources associated to the API"""
        return self._resources

    @property
    def description(self):
        """The description of the API"""
        return self._description

    def __init__(self, name, resources, description):
        """Create a new object that self-document an API

        Args:
          name: The name of the API
          resources: The list of ResourceDescription objects to describe
                     the available resources for the API
          description: A textual description of the API
        """
        self._name = name
        self._resources = resources
        self._description = description


#===============================================================================
# ResourceDescription
#===============================================================================
class ResourceDescription(object):
    """Represents a self descriptive object to document a REST resource
    handler (controller).
    """
    _name = ''
    _url = ''
    _methods = []
    _description = ''

    @property
    def name(self):
        """The name of the resource"""
        return self._name

    @property
    def url(self):
        """The URL where clients can interact with the resource"""
        return self._url

    @url.setter
    def url(self, value):
        self._url = value

    @property
    def methods(self):
        """The list of methods available to interact with the resource"""
        return self._methods

    @methods.setter
    def methods(self, value):
        self._methods = value

    @property
    def description(self):
        """A description for the resource"""
        return self._description

    @description.setter
    def description(self, value):
        self._description = value

    def __init__(self, name, url = '', methods = [], description = ''):
        """Creates a new description for the options to interact with a
        resource.

        Args:
          name: The name of the resource
          url: The url where the clients can interact with the resource
          methods: The list of HttpMethodDescription objects to describe
                   the available methods for the resource
          description: A textual description of the resource
        """
        self._name = name
        self.url = url
        self.methods = methods
        self.description = description

#===============================================================================
# HttpMethodDescription
#===============================================================================
class HttpMethodDescription(object):
    """Represents a self descriptive object to document an HTTP Method"""
    _method = ''
    _parameters = []
    _representations = [RepresentationType.JSON]
    _languages = []
    _description = ''
    _example_uri = ''

    @property
    def method(self):
        """The name of the HTTP method"""
        return self._method

    @property
    def parameters(self):
        """The list of the HttpParameterDescription objects to describe
        the parameters of this HTTP method
        """
        return self._parameters

    @parameters.setter
    def parameters(self, value):
        self._parameters = value

    @property
    def representations(self):
        """The media type representations supported for the method"""
        return self._representations

    @representations.setter
    def representations(self, value):
        self._representations = value

    @property
    def languages(self):
        """The language representations supported for the method"""
        return self._languages

    @languages.setter
    def languages(self, value):
        self._languages = value

    @property
    def description(self):
        """A description of the purpose of the HTTP method"""
        return self._description

    @description.setter
    def description(self, value):
        self._description = value

    @property
    def example_uri(self):
        """URI of a valid example for an HTTP GET over the resource"""
        return self._example_uri

    @example_uri.setter
    def example_uri(self, value):
        self._example_uri = value

    def __init__(self,
                 method,
                 parameters = [],
                 representations=[RepresentationType.JSON],
                 languages = [],
                 description = '',
                 example_uri = ''):
        """
        Creates a new description for an HTTP method with the given arguments

        Args:
          method: The name of the HTTP method to describe
          parameters: A list of HTTPParameterDescription objects with the
                      description of this method's parameters
          representations: The list of Internet Media Types supported as the
                          representations of the resource for the HTTP method.
          languages: The list of supported languages to represent the response
                     for the HTTP method.
          description: A textual description of the method.
          example_uri: The URI of a valid GET example (for GET methods only)
        """
        self._method = method
        self.parameters = parameters
        self.representations = representations
        self.languages = languages
        self.description = description
        self.example_uri = example_uri


#===============================================================================
# HttpParameterDescription
#===============================================================================
class HttpParameterDescription(object):
    """Represents a self descriptive object to document an HTTP parameter
    of an HTTP Method request.
    """
    _name = ''
    _parameter_type = 'string'
    _is_required = False
    _description = ''

    @property
    def name(self):
        """The name of the HTTP parameter"""
        return self._name

    @property
    def parameter_type(self):
        """The type of the HTTP parameter"""
        return self._parameter_type

    @parameter_type.setter
    def parameter_type(self, value):
        self._parameter_type = value

    @property
    def is_required(self):
        """A value indicating if the HTTP parameter is required or not"""
        return self._is_required

    @is_required.setter
    def is_required(self, value):
        self._is_required = value

    @property
    def description(self):
        """A description of the HTTP parameter meaning and purpose"""
        return self._description

    @description.setter
    def description(self, value):
        self._description = value

    def __init__(self, name, parameter_type = 'string', is_required = False, description = ''):
        """
        Creates a new description for an HTTP method parameter with the
        given arguments.

        Args:
          name: The name of the parameter
          parameter_type: The expected 'type' for the parameter
          is_required: True if the parameter is required, False otherwise
          description: A brief description of the parameter meaning and use
        """
        self._name = name
        self.parameter_type = parameter_type
        self.is_required = is_required
        self.description = description

#===============================================================================
# ParameterOptionsJSONRepresentation
#===============================================================================
class ParameterOptionsJSONRepresentation(ResourceJSONRepresentation):
    """The response to show a method parameter"""
    def __init__(self, parameter):
        ResourceJSONRepresentation.__init__(self, 'HttpParameter')
        self.name = parameter.name
        self.type = parameter.parameter_type
        self.required = parameter.is_required
        self.description = parameter.description

#===============================================================================
# MethodOptionsJSONRepresentation
#===============================================================================
class MethodOptionsJSONRepresentation(ResourceJSONRepresentation):
    """The response to show the options for a particular method"""
    def __init__(self, method):
        ResourceJSONRepresentation.__init__(self, 'HttpMethod')
        self.method = method.method
        self.description = method.description
        if method.method == 'GET' and method.example_uri is not None and len(method.example_uri) > 0:
            self.example = ''.join(['http://jsonviewer.stack.hu/#', method.example_uri])
        self.parameters = [ParameterOptionsJSONRepresentation(parameter) for parameter in method.parameters]

#===============================================================================
# ResourceOptionsJSONRepresentation
#===============================================================================
class ResourceOptionsJSONRepresentation(ResourceJSONRepresentation):
    """The response to show the options for a particular resource"""
    def __init__(self, rest_resource_description):
        ResourceJSONRepresentation.__init__(self, 'RESTResource')
        self.name = rest_resource_description.name
        self.url = rest_resource_description.url
        self.description = rest_resource_description.description
        self.methods = [MethodOptionsJSONRepresentation(method) for method in rest_resource_description.methods]

#===============================================================================
# RestApiDocJSONRepresentation
#===============================================================================
class RestApiDocJSONRepresentation(ResourceJSONRepresentation):
    """The response to show the options for the entire API"""
    def __init__(self, api_description):
        ResourceJSONRepresentation.__init__(self, 'RESTAPI')
        self.name = api_description.name
        self.description = api_description.description
        self.resources = [ResourceOptionsJSONRepresentation(resource) for resource in api_description.resources]

